@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v27ter_tx.c,v 1.49 2007/01/03 14:15:36 steveu Exp $
+ * $Id: v27ter_tx.c,v 1.58 2008/03/05 13:38:24 steveu Exp $
  */
 
 /*! \file */
@@ -53,6 +53,14 @@
 
 #include "spandsp/v27ter_tx.h"
 
+#if defined(SPANDSP_USE_FIXED_POINT)
+#include "v27ter_tx_4800_fixed_rrc.h"
+#include "v27ter_tx_2400_fixed_rrc.h"
+#else
+#include "v27ter_tx_4800_floating_rrc.h"
+#include "v27ter_tx_2400_floating_rrc.h"
+#endif
+
 #define CARRIER_NOMINAL_FREQ            1800.0f
 
 /* Segments of the training sequence */
@@ -66,299 +74,6 @@
 #define V27TER_TRAINING_END             (V27TER_TRAINING_SEG_5 + 8)
 #define V27TER_TRAINING_SHUTDOWN_END    (V27TER_TRAINING_END + 32)
 
-/* Raised root cosine pulse shaping; Beta = 0.5; 4 symbols either
-   side of the centre. */
-/* Created with mkshape -r 0.025 0.5 181 -l and then split up */
-#define PULSESHAPER_2400_GAIN           (19.972065748f/20.0f)
-#define PULSESHAPER_2400_COEFF_SETS     20
-static const float pulseshaper_2400[PULSESHAPER_2400_COEFF_SETS][V27TER_TX_FILTER_STEPS] =
-{
-    {
-         0.0050051219f,         /* Filter 0 */
-         0.0107180844f,
-        -0.0150077814f,
-        -0.0750272071f,
-         0.5786341413f,
-         0.5786341413f,
-        -0.0750272071f,
-        -0.0150077814f,
-         0.0107180844f
-    },
-    {
-         0.0036624469f,         /* Filter 1 */
-         0.0131516633f,
-        -0.0107913392f,
-        -0.0957820135f,
-         0.6671466059f,
-         0.4891745311f,
-        -0.0541239470f,
-        -0.0179109014f,
-         0.0079099936f
-    },
-    {
-         0.0020204744f,         /* Filter 2 */
-         0.0150588729f,
-        -0.0053908083f,
-        -0.1154114754f,
-         0.7528295479f,
-         0.4006032722f,
-        -0.0339459430f,
-        -0.0194500407f,
-         0.0048904515f
-    },
-    {
-         0.0001596234f,         /* Filter 3 */
-         0.0163079778f,
-         0.0009858079f,
-        -0.1328632049f,
-         0.8338068363f,
-         0.3146585634f,
-        -0.0152415667f,
-        -0.0196492903f,
-         0.0018247182f
-    },
-    {
-        -0.0018233575f,         /* Filter 4 */
-         0.0167957238f,
-         0.0080554403f,
-        -0.1470417557f,
-         0.9082626683f,
-         0.2329352195f,
-         0.0013822552f,
-        -0.0186004475f,
-        -0.0011283792f
-    },
-    {
-        -0.0038199491f,         /* Filter 5 */
-         0.0164546659f,
-         0.0154676597f,
-        -0.1568448230f,
-         0.9744947974f,
-         0.1568443643f,
-         0.0154698286f,
-        -0.0164532877f,
-        -0.0038242967f
-    },
-    {
-        -0.0057152767f,          /* Filter 6 */
-         0.0152590213f,
-         0.0228163087f,
-        -0.1612020164f,
-         1.0309651039f,
-         0.0875801110f,
-         0.0267201501f,
-        -0.0134037738f,
-        -0.0061394831f
-    },
-    {
-        -0.0073941287f,         /* Filter 7 */
-         0.0132286539f,
-         0.0296547979f,
-        -0.1591148676f,
-         1.0763457753f,
-         0.0260941722f,
-         0.0349842710f,
-        -0.0096808822f,
-        -0.0079766730f
-    },
-    {
-        -0.0087472825f,         /* Filter 8 */
-         0.0104308721f,
-         0.0355146231f,
-        -0.1496966290f,
-         1.1095595051f,
-        -0.0269209682f,
-         0.0402570324f,
-        -0.0055327477f,
-        -0.0092685626f
-    },
-    {
-        -0.0096778115f,         /* Filter 9 */
-         0.0069798134f,
-         0.0399264862f,
-        -0.1322103702f,
-         1.1298123136f,
-        -0.0710400038f,
-         0.0426638320f,
-        -0.0012128224f,
-        -0.0099797659f
-    },
-    {
-        -0.0101070340f,         /* Filter 10 */
-         0.0030333009f,
-         0.0424432507f,
-        -0.1061038872f,
-         1.1366178484f,
-        -0.1061038872f,
-         0.0424432507f,
-         0.0030333009f,
-        -0.0101070340f
-    },
-    {
-        -0.0099797659f,         /* Filter 11 */
-        -0.0012128224f,
-         0.0426638320f,
-        -0.0710400038f,
-         1.1298123136f,
-        -0.1322103702f,
-         0.0399264862f,
-         0.0069798134f,
-        -0.0096778115f
-    },
-    {
-        -0.0092685626f,         /* Filter 12 */
-        -0.0055327477f,
-         0.0402570324f,
-        -0.0269209682f,
-         1.1095595051f,
-        -0.1496966290f,
-         0.0355146231f,
-         0.0104308721f,
-        -0.0087472825f
-    },
-    {
-        -0.0079766730f,         /* Filter 13 */
-        -0.0096808822f,
-         0.0349842710f,
-         0.0260941722f,
-         1.0763457753f,
-        -0.1591148676f,
-         0.0296547979f,
-         0.0132286539f,
-        -0.0073941287f
-    },
-    {
-        -0.0061394831f,         /* Filter 14 */
-        -0.0134037738f,
-         0.0267201501f,
-         0.0875801110f,
-         1.0309651039f,
-        -0.1612020164f,
-         0.0228163087f,
-         0.0152590213f,
-        -0.0057152767f
-    },
-    {
-        -0.0038242967f,         /* Filter 15 */
-        -0.0164532877f,
-         0.0154698286f,
-         0.1568443643f,
-         0.9744947974f,
-        -0.1568448230f,
-         0.0154676597f,
-         0.0164546659f,
-        -0.0038199491f
-    },
-    {
-        -0.0011283792f,         /* Filter 16 */
-        -0.0186004475f,
-         0.0013822552f,
-         0.2329352195f,
-         0.9082626683f,
-        -0.1470417557f,
-         0.0080554403f,
-         0.0167957238f,
-        -0.0018233575f
-    },
-    {
-         0.0018247182f,         /* Filter 17 */
-        -0.0196492903f,
-        -0.0152415667f,
-         0.3146585634f,
-         0.8338068363f,
-        -0.1328632049f,
-         0.0009858079f,
-         0.0163079778f,
-         0.0001596234f
-    },
-    {
-         0.0048904515f,         /* Filter 18 */
-        -0.0194500407f,
-        -0.0339459430f,
-         0.4006032722f,
-         0.7528295479f,
-        -0.1154114754f,
-        -0.0053908083f,
-         0.0150588729f,
-         0.0020204744f
-    },
-    {
-         0.0079099936f,         /* Filter 19 */
-        -0.0179109014f,
-        -0.0541239470f,
-         0.4891745311f,
-         0.6671466059f,
-        -0.0957820135f,
-        -0.0107913392f,
-         0.0131516633f,
-         0.0036624469f
-    },
-};
-
-/* Raised root cosine pulse shaping; Beta = 0.5; 4 symbols either
-   side of the centre. */
-/* Created with mkshape -r 0.1 0.5 45 -l and then split up */
-#define PULSESHAPER_4800_GAIN           (4.9913162900f/5.0f)
-#define PULSESHAPER_4800_COEFF_SETS     5
-static const float pulseshaper_4800[PULSESHAPER_4800_COEFF_SETS][V27TER_TX_FILTER_STEPS] =
-{
-    {
-         0.0020173211f,         /* Filter 0 */
-         0.0150576434f,
-        -0.0053888047f,
-        -0.1154099010f,
-         0.7528286821f,
-         0.4006013374f,
-        -0.0339462085f,
-        -0.0194477281f,
-         0.0048918464f
-    },
-    {
-        -0.0057162575f,         /* Filter 1 */
-         0.0152563286f,
-         0.0228163350f,
-        -0.1612000503f,
-         1.0309660372f,
-         0.0875788553f,
-         0.0267182476f,
-        -0.0134032156f,
-        -0.0061365979f
-    },
-    {
-        -0.0101052019f,         /* Filter 2 */
-         0.0030314952f,
-         0.0424414442f,
-        -0.1061032862f,
-         1.1366196464f,
-        -0.1061032862f,
-         0.0424414442f,
-         0.0030314952f,
-        -0.0101052019f
-    },
-    {
-        -0.0061365979f,         /* Filter 3 */
-        -0.0134032156f,
-         0.0267182476f,
-         0.0875788553f,
-         1.0309660372f,
-        -0.1612000503f,
-         0.0228163350f,
-         0.0152563286f,
-        -0.0057162575f
-    },
-    {
-         0.0048918464f,         /* Filter 4 */
-        -0.0194477281f,
-        -0.0339462085f,
-         0.4006013374f,
-         0.7528286821f,
-        -0.1154099010f,
-        -0.0053888047f,
-         0.0150576434f,
-         0.0020173211f
-    },
-};
-
 static int fake_get_bit(void *user_data)
 {
     return 1;
@@ -404,7 +119,11 @@ static __inline__ int get_scrambled_bit(v27ter_tx_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
+#if defined(SPANDSP_USE_FIXED_POINT)
+static complexi16_t getbaud(v27ter_tx_state_t *s)
+#else
 static complexf_t getbaud(v27ter_tx_state_t *s)
+#endif
 {
     static const int phase_steps_4800[8] =
     {
@@ -414,6 +133,19 @@ static complexf_t getbaud(v27ter_tx_state_t *s)
     {
         0, 2, 6, 4
     };
+#if defined(SPANDSP_USE_FIXED_POINT)
+    static const complexi16_t constellation[8] =
+    {
+        { 1414,    0000},       /*   0deg */
+        { 1000,    1000},       /*  45deg */
+        { 0000,    1414},       /*  90deg */
+        {-1000,    1000},       /* 135deg */
+        {-1414,    0000},       /* 180deg */
+        {-1000,   -1000},       /* 225deg */
+        { 0000,   -1414},       /* 270deg */
+        { 1000,   -1000}        /* 315deg */
+    };
+#else
     static const complexf_t constellation[8] =
     {
         { 1.414f,  0.0f},       /*   0deg */
@@ -425,6 +157,7 @@ static complexf_t getbaud(v27ter_tx_state_t *s)
         { 0.0f,   -1.414f},     /* 270deg */
         { 1.0f,   -1.0f}        /* 315deg */
     };
+#endif
     int bits;
 
     if (s->in_training)
@@ -442,7 +175,11 @@ static complexf_t getbaud(v27ter_tx_state_t *s)
                 if (s->training_step <= V27TER_TRAINING_SEG_3)
                 {
                     /* Segment 2: Silence */
-                    return complex_setf(0.0, 0.0);
+#if defined(SPANDSP_USE_FIXED_POINT)
+                    return complex_seti16(0, 0);
+#else
+                    return complex_setf(0.0f, 0.0f);
+#endif
                 }
                 /* Segment 3: Regular reversals... */
                 s->constellation_state = (s->constellation_state + 4) & 7;
@@ -491,8 +228,13 @@ static complexf_t getbaud(v27ter_tx_state_t *s)
 
 int v27ter_tx(v27ter_tx_state_t *s, int16_t amp[], int len)
 {
+#if defined(SPANDSP_USE_FIXED_POINT)
+    complexi_t x;
+    complexi_t z;
+#else
     complexf_t x;
     complexf_t z;
+#endif
     int i;
     int sample;
 
@@ -517,17 +259,32 @@ int v27ter_tx(v27ter_tx_state_t *s, int16_t amp[], int len)
                     s->rrc_filter_step = 0;
             }
             /* Root raised cosine pulse shaping at baseband */
-            x.re = 0.0f;
-            x.im = 0.0f;
+#if defined(SPANDSP_USE_FIXED_POINT)
+            x = complex_seti(0, 0);
             for (i = 0;  i < V27TER_TX_FILTER_STEPS;  i++)
             {
-                x.re += pulseshaper_4800[4 - s->baud_phase][i]*s->rrc_filter[i + s->rrc_filter_step].re;
-                x.im += pulseshaper_4800[4 - s->baud_phase][i]*s->rrc_filter[i + s->rrc_filter_step].im;
+                x.re += (int32_t) pulseshaper_4800[PULSESHAPER_4800_COEFF_SETS - 1 - s->baud_phase][i]*(int32_t) s->rrc_filter[i + s->rrc_filter_step].re;
+                x.im += (int32_t) pulseshaper_4800[PULSESHAPER_4800_COEFF_SETS - 1 - s->baud_phase][i]*(int32_t) s->rrc_filter[i + s->rrc_filter_step].im;
+            }
+            /* Now create and modulate the carrier */
+            x.re >>= 14;
+            x.im >>= 14;
+            z = dds_complexi(&(s->carrier_phase), s->carrier_phase_rate);
+            /* Don't bother saturating. We should never clip. */
+            i = (x.re*z.re - x.im*z.im) >> 15;
+            amp[sample] = (int16_t) ((i*s->gain_4800) >> 15);
+#else
+            x = complex_setf(0.0f, 0.0f);
+            for (i = 0;  i < V27TER_TX_FILTER_STEPS;  i++)
+            {
+                x.re += pulseshaper_4800[PULSESHAPER_4800_COEFF_SETS - 1 - s->baud_phase][i]*s->rrc_filter[i + s->rrc_filter_step].re;
+                x.im += pulseshaper_4800[PULSESHAPER_4800_COEFF_SETS - 1 - s->baud_phase][i]*s->rrc_filter[i + s->rrc_filter_step].im;
             }
             /* Now create and modulate the carrier */
             z = dds_complexf(&(s->carrier_phase), s->carrier_phase_rate);
             /* Don't bother saturating. We should never clip. */
             amp[sample] = (int16_t) rintf((x.re*z.re - x.im*z.im)*s->gain_4800);
+#endif
         }
     }
     else
@@ -543,17 +300,32 @@ int v27ter_tx(v27ter_tx_state_t *s, int16_t amp[], int len)
                     s->rrc_filter_step = 0;
             }
             /* Root raised cosine pulse shaping at baseband */
-            x.re = 0.0f;
-            x.im = 0.0f;
+#if defined(SPANDSP_USE_FIXED_POINT)
+            x = complex_seti(0, 0);
             for (i = 0;  i < V27TER_TX_FILTER_STEPS;  i++)
             {
-                x.re += pulseshaper_2400[19 - s->baud_phase][i]*s->rrc_filter[i + s->rrc_filter_step].re;
-                x.im += pulseshaper_2400[19 - s->baud_phase][i]*s->rrc_filter[i + s->rrc_filter_step].im;
+                x.re += (int32_t) pulseshaper_2400[PULSESHAPER_2400_COEFF_SETS - 1 - s->baud_phase][i]*(int32_t) s->rrc_filter[i + s->rrc_filter_step].re;
+                x.im += (int32_t) pulseshaper_2400[PULSESHAPER_2400_COEFF_SETS - 1 - s->baud_phase][i]*(int32_t) s->rrc_filter[i + s->rrc_filter_step].im;
+            }
+            /* Now create and modulate the carrier */
+            x.re >>= 14;
+            x.im >>= 14;
+            z = dds_complexi(&(s->carrier_phase), s->carrier_phase_rate);
+            /* Don't bother saturating. We should never clip. */
+            i = (x.re*z.re - x.im*z.im) >> 15;
+            amp[sample] = (int16_t) ((i*s->gain_2400) >> 15);
+#else
+            x = complex_setf(0.0f, 0.0f);
+            for (i = 0;  i < V27TER_TX_FILTER_STEPS;  i++)
+            {
+                x.re += pulseshaper_2400[PULSESHAPER_2400_COEFF_SETS - 1 - s->baud_phase][i]*s->rrc_filter[i + s->rrc_filter_step].re;
+                x.im += pulseshaper_2400[PULSESHAPER_2400_COEFF_SETS - 1 - s->baud_phase][i]*s->rrc_filter[i + s->rrc_filter_step].im;
             }
             /* Now create and modulate the carrier */
             z = dds_complexf(&(s->carrier_phase), s->carrier_phase_rate);
             /* Don't bother saturating. We should never clip. */
             amp[sample] = (int16_t) rintf((x.re*z.re - x.im*z.im)*s->gain_2400);
+#endif
         }
     }
     return sample;
@@ -565,8 +337,13 @@ void v27ter_tx_power(v27ter_tx_state_t *s, float power)
     float l;
 
     l = powf(10.0f, (power - DBM0_MAX_POWER)/20.0f)*32768.0f;
+#if defined(SPANDSP_USE_FIXED_POINT)
+    s->gain_2400 = 16.0f*1.024f*(32767.0f/28828.51f)*l/PULSESHAPER_2400_GAIN;
+    s->gain_4800 = 16.0f*1.024f*(32767.0f/28828.46f)*l/PULSESHAPER_4800_GAIN;
+#else
     s->gain_2400 = l/PULSESHAPER_2400_GAIN;
     s->gain_4800 = l/PULSESHAPER_4800_GAIN;
+#endif
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -584,7 +361,11 @@ int v27ter_tx_restart(v27ter_tx_state_t *s, int rate, int tep)
     if (rate != 4800  &&  rate != 2400)
         return -1;
     s->bit_rate = rate;
+#if defined(SPANDSP_USE_FIXED_POINT)
+    memset(s->rrc_filter, 0, sizeof(s->rrc_filter));
+#else
     cvec_zerof(s->rrc_filter, sizeof(s->rrc_filter)/sizeof(s->rrc_filter[0]));
+#endif
     s->rrc_filter_step = 0;
     s->scramble_reg = 0x3C;
     s->scrambler_pattern_count = 0;
@@ -606,6 +387,8 @@ v27ter_tx_state_t *v27ter_tx_init(v27ter_tx_state_t *s, int rate, int tep, get_b
             return NULL;
     }
     memset(s, 0, sizeof(*s));
+    span_log_init(&s->logging, SPAN_LOG_NONE, NULL);
+    span_log_set_protocol(&s->logging, "V.27ter TX");
     s->get_bit = get_bit;
     s->user_data = user_data;
     s->carrier_phase_rate = dds_phase_ratef(CARRIER_NOMINAL_FREQ);
@@ -615,7 +398,7 @@ v27ter_tx_state_t *v27ter_tx_init(v27ter_tx_state_t *s, int rate, int tep, get_b
 }
 /*- End of function --------------------------------------------------------*/
 
-int v27ter_tx_release(v27ter_tx_state_t *s)
+int v27ter_tx_free(v27ter_tx_state_t *s)
 {
     free(s);
     return 0;
